/* R_RANDOM.H - header file for R_RANDOM.C
 */

/* Copyright (C) RSA Laboratories, a division of RSA Data Security,
     Inc., created 1991. All rights reserved.
 */

int R_GenerateBytes PROTO_LIST
  ((unsigned char *, unsigned int, R_RANDOM_STRUCT *));
